// MainView.cpp : implementazione della classe CMainView
//

#include "stdafx.h"
#include "Paroliamo.h"

#include "MainDoc.h"
#include "MainView.h"

#include <AgtSvr_i.c>
#include <mmsystem.h>

#pragma comment(lib, "winmm.lib")

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CMainView

IMPLEMENT_DYNCREATE(CMainView, CView)

BEGIN_MESSAGE_MAP(CMainView, CView)
	ON_WM_DESTROY()
	ON_MESSAGE(WM_SPEAK_COMPLETE, OnSpeakComplete)
	ON_WM_LBUTTONDOWN()
END_MESSAGE_MAP()

// costruzione/eliminazione di CMainView

CMainView::CMainView()
{
	m_pAgentEx = NULL;
	m_pChar = NULL;
	m_pEvent = NULL;
	m_lCharID = 0;
	m_lRequestID = 0;
	m_lEventID = 0;
	m_nEsatte = 0;
	m_nCurLetter = -1;
	m_nNow = -1;
	m_nFail = 0;
	
	m_cLetter[0] = 'A';
	m_cLetter[1] = 'B';
	m_cLetter[2] = 'C';
	m_cLetter[3] = 'D';
	m_cLetter[4] = 'E';
	m_cLetter[5] = 'F';
	m_cLetter[6] = 'G';
	m_cLetter[7] = 'H';
	m_cLetter[8] = 'I';
	m_cLetter[9] = 'L';
	m_cLetter[10] = 'M';
	m_cLetter[11] = 'N';
	m_cLetter[12] = 'O';
	m_cLetter[13] = 'P';
	m_cLetter[14] = 'Q';
	m_cLetter[15] = 'R';
	m_cLetter[16] = 'S';
	m_cLetter[17] = 'T';
	m_cLetter[18] = 'U';
	m_cLetter[19] = 'V';
	m_cLetter[20] = 'Z';

	m_nOrder[0] = 0;
	m_nOrder[1] = 4;
	m_nOrder[2] = 8;
	m_nOrder[3] = 12;
	m_nOrder[4] = 18;

	for (int i = 5; i < 26; ++i)
		m_nOrder[i] = i - 5;
}

CMainView::~CMainView()
{
}

BOOL CMainView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

// disegno di CMainView

void CMainView::OnDraw(CDC* pDC)
{
	CMainDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect rect;
	GetClientRect(&rect);

	if (!m_bitImage.IsNull())
	{
        int mode = pDC->SetStretchBltMode(COLORONCOLOR);
		m_bitImage.Draw(pDC->GetSafeHdc(), rect);
		pDC->SetStretchBltMode(mode);
	}

	if (m_rc[0].IsRectEmpty())
	{
		int nHeight = rect.Height() / 4;
		int nWidth = 300;

		m_rc[0].left = 0;
		m_rc[0].right = m_rc[0].left + nWidth;
		m_rc[0].top = 0;
		m_rc[0].bottom = m_rc[0].top + nHeight;
		m_rect = m_rc[0];

		m_rc[1].left = 0;
		m_rc[1].right = m_rc[1].left + nWidth;
		m_rc[1].top = m_rc[0].bottom;
		m_rc[1].bottom = m_rc[1].top + nHeight;
		
		m_rc[2].left = 0;
		m_rc[2].right = m_rc[2].left + nWidth;
		m_rc[2].top = m_rc[1].bottom;
		m_rc[2].bottom = m_rc[2].top + nHeight;
		
		m_rc[3].left = 0;
		m_rc[3].right = m_rc[3].left + nWidth;
		m_rc[3].top = m_rc[2].bottom;
		m_rc[3].bottom = m_rc[3].top + nHeight;
		m_rect.bottom = m_rc[3].bottom;
	}
	
	if (m_nCurLetter >= 0)
	{
		if (m_nOrder[m_nCurLetter] >= 0)
		{
            DrawLetters(pDC, m_nOrder[m_nCurLetter], RGB(255, 100, 100), m_rc[0]);
			DrawLetters(pDC, m_nOrder[m_nCurLetter] + 21, RGB(100, 255, 100), m_rc[1]);
			DrawLetters(pDC, m_nOrder[m_nCurLetter] + 42, RGB(150, 150, 255), m_rc[2]);
			DrawLetters(pDC, m_nOrder[m_nCurLetter] + 63, RGB(255, 255, 100), m_rc[3]);
		}
	}
}

void CMainView::DrawLetters(CDC* pDC, int nLetter, COLORREF crBack, CRect rect)
{
	CPoint pt = rect.CenterPoint();
	pDC->FillSolidRect(&rect, crBack);
	pt.x -= 25;
	pt.y -= 25;
	m_imgLetter.Draw(pDC, nLetter, pt, ILD_TRANSPARENT);
}

// diagnostica di CMainView

#ifdef _DEBUG
void CMainView::AssertValid() const
{
	CView::AssertValid();
}

void CMainView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMainDoc* CMainView::GetDocument() const // la versione non debug  inline.
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMainDoc)));
	return (CMainDoc*)m_pDocument;
}
#endif //_DEBUG


// gestori di messaggi di CMainView

void CMainView::OnInitialUpdate()
{
	CView::OnInitialUpdate();
	
	CComBSTR szBuffer;
	VARIANT vPath;
	VariantInit(&vPath);
	
	vPath.vt = VT_BSTR;
	vPath.bstrVal = SysAllocString(L"Peedy.acs");

	HRESULT hr = CoCreateInstance(CLSID_AgentServer, NULL, CLSCTX_SERVER, IID_IAgentEx,
		(LPVOID*)&m_pAgentEx);

	if (SUCCEEDED(hr))
	{
		m_pEvent = new CNotifyEx;
		m_pEvent->m_pView = this;
		m_pEvent->AddRef();
		hr = m_pAgentEx->Register((IUnknown*)m_pEvent, &m_lEventID);
	}

	if (SUCCEEDED(hr))
	{
		hr = m_pAgentEx->Load(vPath, &m_lCharID, &m_lRequestID);
	}
	
	if (SUCCEEDED(hr))
	{
		hr = m_pAgentEx->GetCharacterEx(m_lCharID, &m_pChar);
	}
	
	if (SUCCEEDED(hr))
	{
		CRect rect;
		GetClientRect(&rect);

		hr = m_pChar->SetPosition(rect.right - 200, rect.bottom - 200);
		if (FAILED(hr))
		{
			AfxMessageBox(_T("Impossibile posizionare Peedy!"), MB_ICONSTOP);
			PostQuitMessage(0);
		}

		hr = m_pChar->Show(FALSE, &m_lRequestID);
		if (FAILED(hr))
		{
			AfxMessageBox(_T("Impossibile mostrare Peedy!"), MB_ICONSTOP);
			PostQuitMessage(0);
		}
		
		char c = '"';
		CString buffer;
		
		szBuffer = L"Greet";
		hr = m_pChar->Play(szBuffer, &m_lRequestID);
		if (FAILED(hr))
		{
			AfxMessageBox(_T("Azione 1 impossibile Peedy!"), MB_ICONSTOP);
			PostQuitMessage(0);
		}
        szBuffer = L"RestPose";
		hr = m_pChar->Play(szBuffer, &m_lRequestID);
		if (FAILED(hr))
		{
			AfxMessageBox(_T("Azione 2 impossibile Peedy!"), MB_ICONSTOP);
			PostQuitMessage(0);
		}
		buffer.Format(_T("Ciao \\map=%ciacopo%c=%cJacopo,%c\\ io sono Portobello, e ti aiuter a memorizzare, le lettere \
			dell'alfabeto"), c, c, c, c);
        szBuffer = buffer;        
		hr = m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
		if (FAILED(hr))
		{
			AfxMessageBox(_T("Azione 3 impossibile Peedy!"), MB_ICONSTOP);
			PostQuitMessage(0);
		}
		szBuffer = L"Vedrai, come  facile imparare con me!";
		hr = m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
		if (FAILED(hr))
		{
			AfxMessageBox(_T("Azione 4 impossibile Peedy!"), MB_ICONSTOP);
			PostQuitMessage(0);
		}
	}
	VariantClear(&vPath);

	CBitmap bmp;
	bmp.LoadBitmap(IDB_BITMAP_LETTERE);
	m_imgLetter.Create(50, 50, ILC_COLOR4|ILC_MASK, 84, 0);
	m_imgLetter.Add(&bmp, RGB(192, 192, 192));
	bmp.DeleteObject();

	m_bitImage.LoadFromResource(AfxGetInstanceHandle(), IDB_BITMAP_JACOPO);
	Invalidate();
}

void CMainView::OnDestroy()
{
	m_pChar->StopAll(STOP_TYPE_ALL);
	m_pChar->Hide(FALSE, NULL);
	
	if (m_pEvent != NULL)
		delete m_pEvent;

	m_pChar->Release();
	m_pChar = NULL;
	m_pAgentEx->Unload(m_lCharID);
	m_pAgentEx->Release();
	m_pAgentEx = NULL;

	if (!m_bitImage.IsNull())
		m_bitImage.Destroy();

	CView::OnDestroy();
}

LRESULT CMainView::OnSpeakComplete(WPARAM wParam, LPARAM lParam)
{
	if (m_pEvent != NULL)
	{
		if (wParam == 111) // inizio
		{
			m_nCurLetter = 0;
            InvalidateRect(&m_rect);
			ExplainLetter(m_nOrder[m_nCurLetter]);
			AnotherOne();
		}
	}
	return 0;
}

void CMainView::OnLButtonDown(UINT nFlags, CPoint point)
{
	if (m_lRequestID > 110 && m_rect.PtInRect(point))
	{
        if (m_rc[m_nNow].PtInRect(point))
		{
			CString buffer;
			++m_nEsatte;
			buffer.Format(_T("\\Vol=50000\\Bravissimo!!! %d risposte esatte!"), m_nEsatte);
			CComBSTR szBuffer = L"Congratulate";
			m_pChar->Play(szBuffer, &m_lRequestID);
			szBuffer = buffer;
			m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
			szBuffer = L"RestPose";
			m_pChar->Play(szBuffer, &m_lRequestID);
			if (m_nEsatte < 10)
			{
				m_nFail = 0;
                AnotherOne();
			}
			else
			{
				++m_nCurLetter;
				if (m_nCurLetter > 25)
					m_nCurLetter = 0;
				
				CString buffer;
				char c = '"';
		
				szBuffer = L"Write";
				m_pChar->Play(szBuffer, &m_lRequestID);
				buffer.Format(_T("\\Vol=50000\\Bravissimo \\map=%ciacopo%c=%cJacopo%c\\ ce l'hai fatta!!!"), c, c, c, c);
				szBuffer = buffer;
				m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
				szBuffer = L"Sai che faccio ora? Ti scrivo tra i pi bravi!|Sai cosa facciamo? Ti metto una nota di merito|Incredibile! Ti nomino cavaliere delle elementari.";
				m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
				szBuffer = L"WriteReturn";
				m_pChar->Play(szBuffer, &m_lRequestID);

				buffer.Format(_T("Ed ora, la lettera \\Pau=200\\%c"), m_cLetter[m_nOrder[m_nCurLetter]]);
				CComBSTR szBuffer = buffer;
				m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
				InvalidateRect(&m_rect);
				ExplainLetter(m_nOrder[m_nCurLetter]);
				m_nEsatte = 0;
				AnotherOne();
			}
		}
		else
		{
			++m_nFail;
			CComBSTR szBuffer;
			
			if (m_nFail > 0 && m_nFail <= 3)
                szBuffer = L"No, questa non  la lettera giusta!|No, che sbadato!!!|No, riprova!!";
			else if (m_nFail > 3 && m_nFail <= 6)
				szBuffer = L"No, ma che sei di coccio?|No, che ci rifai?|No, testina!";
			else if (m_nFail > 6 && m_nFail <= 9)
				szBuffer = L"No, sei proprio un somarello!|No, che asino!!!!|No, Asinello!";
			else if (m_nFail <= 0)
				szBuffer = L"No riproviamoci!|No, non ti distrarre!|No, pensaci bene!";
			else
			{
				szBuffer = L"Vai al diavolo, gioco finito!|Vai a quel paese, gioco finito!";
				m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
				_sleep(5000);
				PostQuitMessage(0);
				return;
			}
			m_pChar->Speak(szBuffer, NULL, &m_lRequestID);

			if (!m_strLastQuestion.IsEmpty())
			{
				CString buffer;
				buffer.Format(_T("Ti avevo chiesto: %s"), m_strLastQuestion);
				szBuffer = buffer;
				m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
			}
		}
	}
	
	CView::OnLButtonDown(nFlags, point);
}

void CMainView::AnotherOne()
{
	srand((unsigned int)time(NULL));
	m_nNow = rand() % 4;
	--m_nNow;

	CString temp;
	switch (m_nNow)
	{
	case 0:
		temp = _T("maiuscola, scritta in stampatello");
		break;
	case 1:
        temp = _T("minuscola, scritta in stampatello");		
		break;
	case 2:
		temp = _T("maiuscola, scritta in corsivo");
		break;
	case 3:
		temp = _T("minuscola, scritta in corsivo");
		break;
	default:
		m_nNow = 3;
		temp = _T("minuscola, scritta in corsivo");
	}
    CString buffer;
	buffer.Format(_T("Clicca sulla \\Pau=300\\%c,\\Pau=200\\ %s"), m_cLetter[m_nOrder[m_nCurLetter]], temp);

	m_strLastQuestion = buffer;

	CComBSTR szBuffer = buffer;
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
}

void CMainView::ExplainLetter(int nLetter)
{
	CRect rect;
	GetClientRect(&rect);

	CString buffer; CComBSTR szBuffer;
	buffer.Format(_T("Alla tua sinistra sono apparsi i quattro modi per scrivere una %c\\Pau=200\\"), m_cLetter[nLetter]);
	szBuffer = buffer;
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
	m_pChar->MoveTo((short)m_rc[0].right, (short)(m_rc[0].top + (m_rc[0].Height() / 2)), 2000, &m_lRequestID);
	m_pChar->GestureAt((short)m_rc[0].right, (short)(m_rc[0].top + (m_rc[0].Height() / 2)), &m_lRequestID);

	buffer.Format(_T("La prima forma, nella casella rossa,  una \\Pau=300\\%c,\\Pau=200\\ maiuscola, scritta in stampatello!"), m_cLetter[nLetter]);
	szBuffer = buffer;
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
	m_pChar->MoveTo((short)m_rc[1].right, (short)(m_rc[1].top + (m_rc[1].Height() / 2)), 2000, &m_lRequestID);
	m_pChar->GestureAt((short)m_rc[1].right, (short)(m_rc[1].top + (m_rc[1].Height() / 2)), &m_lRequestID);

	buffer.Format(_T("La seconda forma, nella casella verde,  una \\Pau=300\\%c,\\Pau=200\\ minuscola, scritta in stampatello!"), m_cLetter[nLetter]);
	szBuffer = buffer;
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
	m_pChar->MoveTo((short)m_rc[2].right, (short)(m_rc[2].top + (m_rc[2].Height() / 2)), 2000, &m_lRequestID);
	m_pChar->GestureAt((short)m_rc[2].right, (short)(m_rc[2].top + (m_rc[2].Height() / 2)), &m_lRequestID);

	buffer.Format(_T("La terza forma, nella casella azzurra,  una \\Pau=300\\%c,\\Pau=200\\ maiuscola, scritta in corsivo!"), m_cLetter[nLetter]);
	szBuffer = buffer;
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
	m_pChar->MoveTo((short)m_rc[3].right, (short)(m_rc[3].top + (m_rc[3].Height() / 2)), 2000, &m_lRequestID);
	m_pChar->GestureAt((short)m_rc[3].right, (short)(m_rc[3].top + (m_rc[3].Height() / 2)), &m_lRequestID);
		
	buffer.Format(_T("La quarta ed ultima forma, nella casella gialla,  una \\Pau=300\\%c,\\Pau=200\\ minuscola, scritta in corsivo!"), m_cLetter[nLetter]);
	szBuffer = buffer;
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
	m_pChar->MoveTo((short)(rect.right - 200), (short)(rect.bottom - 200), 3000, &m_lRequestID);
	szBuffer = L"Hai visto bene?\\Pau=500\\Ora, guarda le lettere, e prova a cliccare con il mouse, sulla lettera che ti dir.\\Pau=500\\ Pronto?\\Pau=1000\\ Vai!!!";
	m_pChar->Speak(szBuffer, NULL, &m_lRequestID);
}
