#include "StdAfx.h"
#include "NotifyEx.h"

STDMETHODIMP CNotifyEx::QueryInterface(REFIID riid, LPVOID* ppvObject)
{
	if (riid == IID_IUnknown || riid == IID_IAgentNotifySinkEx)
	{
		*ppvObject = this;
		AddRef();
		return NOERROR;
	}
	return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG) CNotifyEx::AddRef()
{
	return ++m_dwRef;
}

STDMETHODIMP_(ULONG) CNotifyEx::Release()
{
	if (--m_dwRef <= 0)
	{
		delete this;
	}
	return m_dwRef;
}

STDMETHODIMP CNotifyEx::GetTypeInfoCount(UINT* pctinfo)
{
	*pctinfo = 1;
	return NOERROR;
}

STDMETHODIMP CNotifyEx::GetTypeInfo(UINT iTInfo, LCID lcid, ITypeInfo** ppTInfo)
{
	HRESULT hr;
	ITypeLib *pLib;
	
	*ppTInfo = NULL;
	if (iTInfo != 0)
		return TYPE_E_ELEMENTNOTFOUND;
	if (ppTInfo == NULL)
		return E_POINTER;
	if ((PRIMARYLANGID(lcid) != LANG_NEUTRAL) &&  (PRIMARYLANGID(lcid) != LANG_ITALIAN))
		return DISP_E_UNKNOWNLCID;
	hr = LoadRegTypeLib(LIBID_AgentServerObjects, 1, 0, PRIMARYLANGID(lcid), &pLib);
	if (FAILED(hr))
		return hr;
	hr = pLib->GetTypeInfoOfGuid(IID_IAgentNotifySinkEx, ppTInfo);
	pLib->Release();
	if (FAILED(hr))
		return hr;
	(*ppTInfo)->AddRef();
	return NOERROR;
}

STDMETHODIMP CNotifyEx::GetIDsOfNames(REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId)
{
	HRESULT hr;
	ITypeInfo *pInfo;
	if (riid != IID_NULL)
		return ResultFromScode(DISP_E_UNKNOWNINTERFACE);
	hr = GetTypeInfo(0, lcid, &pInfo);
	if (FAILED(hr))
		return hr;
	hr = pInfo->GetIDsOfNames(rgszNames, cNames, rgDispId);
	pInfo->Release();
	return hr;
}

STDMETHODIMP CNotifyEx::Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
{
	HRESULT hr;
	ITypeInfo *pInfo;
	if (riid != IID_NULL)
		return DISP_E_UNKNOWNINTERFACE;
	hr = GetTypeInfo(0, lcid, &pInfo);
	if (FAILED(hr))
		return hr;
	SetErrorInfo(0L, NULL);
	hr = pInfo->Invoke(this, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
	pInfo->Release();
	return hr;
}

STDMETHODIMP CNotifyEx::Command(long dwCommandID, IUnknown* punkUserInput)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::ActivateInputState(long dwCharID, long bActivated)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::Restart()
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::Shutdown()
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::VisibleState(long dwCharID, long bVisible, long dwCause)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::Click(long dwCharID, short fwKeys, long x, long y)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::DblClick(long dwCharID, short fwKeys, long x, long y)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::DragStart(long dwCharID, short fwKeys, long x, long y)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::DragComplete(long dwCharID, short fwKeys, long x, long y)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::RequestStart(long dwRequestID)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::RequestComplete(long dwRequestID, long hrStatus)
{
	if (m_pView != NULL)
	{
        m_pView->SendMessage(WM_SPEAK_COMPLETE, dwRequestID);
	}

	return NOERROR;
}

STDMETHODIMP CNotifyEx::BookMark(long dwBookMarkID)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::Idle(long dwCharID, long bStart)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::Move(long dwCharID, long x, long y, long dwCause)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::Size(long dwCharID, long lWidth, long lHeight)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::BalloonVisibleState(long dwCharID, long bVisible)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::HelpComplete(long dwCharID, long dwCommandID, long dwCause)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::ListeningState(long dwCharID, long bListening, long dwCause)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::DefaultCharacterChange(BSTR bszGUID)
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::AgentPropertyChange()
{
	return NOERROR;
}

STDMETHODIMP CNotifyEx::ActiveClientChange(long dwCharID, long lStatus)
{
	return NOERROR;
}